window.Gleam.initSearch([{"doc":"gleam_otp","title":"gleam_otp","content":"# Gleam OTP\n\n&lt;a href=&quot;https://github.com/gleam-lang/otp/releases&quot;&gt;&lt;img src=&quot;https://img.shields.io/github/release/gleam-lang/otp&quot; alt=&quot;GitHub release&quot;&gt;&lt;/a&gt;\n&lt;a href=&quot;https://discord.gg/Fm8Pwmy&quot;&gt;&lt;img src=&quot;https://img.shields.io/discord/768594524158427167?color=blue&quot; alt=&quot;Discord chat&quot;&gt;&lt;/a&gt;\n![CI](https://github.com/gleam-lang/otp/workflows/test/badge.svg?branch=main)\n\nA Gleam library for building fault tolerant multi-core programs using the\nactor model. It is compatible with Erlang&#39;s OTP framework.\n\nGleam’s actor system is built with a few primary goals:\n\n- Full type safety of actors and messages.\n- Be compatible with Erlang’s OTP actor framework.\n- Provide fault tolerance and self-healing through supervisors.\n- Have equivalent performance to Erlang’s OTP.\n\nThis library documents its abstractions and functionality, but you may also wish\nto read the documentation or other material on Erlang’s OTP framework to get a\nfuller understanding of OTP, the problems it solves, and and the motivations for\nits design.\n\nNot all Erlang/OTP functionality is included in this library. Some is not\npossible to represent in a type safe way, so it is not included. Other features\nare still in development, such as further process supervision strategies.\n\n## Usage\n\nAdd this library to your Gleam project.\n\n```shell\ngleam add gleam_otp\n```\n\n## Actor hierarchy\n\nThis library provides several different types of actor that can be used in\nGleam programs.\n\n### Process\n\nThe process is the lowest level building block of OTP, all other actors are\nbuilt on top of processes either directly or indirectly. Typically this\nabstraction would not be used very often in Gleam applications, favour\nother actor types that provide more functionality.\n\nGleam&#39;s [process](https://hexdocs.pm/gleam_erlang/gleam/erlang/process.html) module is defined in the `gleam_erlang` library.\n\n### Actor\n\nThe `actor` is the most commonly used process type in Gleam and serves as a good\nbuilding block for other abstractions. Like Erlang&#39;s `gen_server` it handles\nOTP&#39;s system messages automatically to enable OTP&#39;s debugging and tracing\nfunctionality.\n\n[Documentation](https://hexdocs.pm/gleam_otp/gleam/otp/actor.html)\n\n### Task\n\nA task is a kind of process that performs a single task and then shuts down.\nCommonly tasks are used to convert sequential code into concurrent code by\nperforming computation in another process.\n\n- [gleam/otp/task](https://hexdocs.pm/gleam_otp/gleam/otp/task.html)\ndocumentation.\n\n### Supervisor\n\nSupervisors is a process that starts and then supervises a group of processes,\nrestarting them if they crash. Supervisors can start other supervisors,\nresulting in a hierarchical process structure called a supervision tree,\nproviding fault tolerance to a Gleam application.\n\n- [gleam/otp/static_supervisor](https://hexdocs.pm/gleam_otp/gleam/otp/static_supervisor.html) documentation.\n- [gleam/otp/supervisor](https://hexdocs.pm/gleam_otp/gleam/otp/supervisor.html)\n  documentation.\n\n## Limitations and known issues\n\nThis library does not currently replicate all of the Erlang/OTP functionality.\nSome limitations include:\n\n- There is no support for named processes. They are untyped global mutable\n  variables which may be uninitialized, more research is needed to find a\n  suitable type safe alternative.\n- There are relatively few actor abstractions provided by this library. More\n  will be added in the future.\n- Actors do not yet support all OTP system messages. Unsupported messages are\n  dropped.\n- Supervisors do not yet support different shutdown periods per child. In\n  practice this means that children that are supervisors do not get an\n  unlimited amount of time to shut down, as is expected in Erlang or Elixir.\n- This library has not seen much testing compared to the Erlang OTP\n  libraries, both in terms of unit tests and real world testing in\n  applications.\n","url":"index.html"},{"doc":"gleam/otp/system","title":"DebugOption","content":"pub type DebugOption {\n  NoDebug\n}\n\nNoDebug\n\n\nSynonyms:\ngleam/otp/system.DebugOption\ngleam/otp/system DebugOption","url":"gleam/otp/system.html#DebugOption"},{"doc":"gleam/otp/system","title":"DebugState","content":"pub type DebugState\n\n\nSynonyms:\ngleam/otp/system.DebugState\ngleam/otp/system DebugState","url":"gleam/otp/system.html#DebugState"},{"doc":"gleam/otp/system","title":"Mode","content":"pub type Mode {\n  Running\n  Suspended\n}\n\nRunning\n Currently handling message as normal.\n\nSuspended\n Termporarily not handling messages, other than system messages.\n\nSynonyms:\ngleam/otp/system.Mode\ngleam/otp/system Mode","url":"gleam/otp/system.html#Mode"},{"doc":"gleam/otp/system","title":"StatusInfo","content":"pub type StatusInfo {\n  StatusInfo(\n    module: Atom,\n    parent: Pid,\n    mode: Mode,\n    debug_state: DebugState,\n    state: Dynamic,\n  )\n}\n\nStatusInfo(\n  module: Atom,\n  parent: Pid,\n  mode: Mode,\n  debug_state: DebugState,\n  state: Dynamic,\n)\n\n\nSynonyms:\ngleam/otp/system.StatusInfo\ngleam/otp/system StatusInfo","url":"gleam/otp/system.html#StatusInfo"},{"doc":"gleam/otp/system","title":"SystemMessage","content":"pub type SystemMessage {\n  Resume(fn() -&gt; Nil)\n  Suspend(fn() -&gt; Nil)\n  GetState(fn(Dynamic) -&gt; Nil)\n  GetStatus(fn(StatusInfo) -&gt; Nil)\n}\n\nResume(fn() -&gt; Nil)\n\n\nSuspend(fn() -&gt; Nil)\n\n\nGetState(fn(Dynamic) -&gt; Nil)\n\n\nGetStatus(fn(StatusInfo) -&gt; Nil)\n\n\nSynonyms:\ngleam/otp/system.SystemMessage\ngleam/otp/system SystemMessage","url":"gleam/otp/system.html#SystemMessage"},{"doc":"gleam/otp/system","title":"debug_state","content":"pub fn debug_state(a: List(DebugOption)) -&gt; DebugState\n\nSynonyms:\ngleam/otp/system.debug_state\ngleam/otp/system debug_state","url":"gleam/otp/system.html#debug_state"},{"doc":"gleam/otp/system","title":"get_state","content":"pub fn get_state(from from: Pid) -&gt; Dynamic\n Get the state of a given OTP compatible process. This function is only\n intended for debugging.\n\n Requires Erlang/OTP 26.1 or newer, as the underlying interface changed\n in [OTP-18633][1] from a literal type to a result type.\n\n For more information see the [Erlang documentation][2].\n\n [1]: https://www.erlang.org/patches/otp-26.1#stdlib-5.1\n [2]: https://erlang.org/doc/man/sys.html#get_state-1\n\nSynonyms:\ngleam/otp/system.get_state\ngleam/otp/system get_state","url":"gleam/otp/system.html#get_state"},{"doc":"gleam/otp/system","title":"resume","content":"pub fn resume(pid: Pid) -&gt; Nil\n Request a suspended OTP compatible process to result, causing it to handle\n all messages rather than only system messages.\n\n For more information see the [Erlang documentation][1].\n\n [1]: https://erlang.org/doc/man/sys.html#resume-1\n\nSynonyms:\ngleam/otp/system.resume\ngleam/otp/system resume","url":"gleam/otp/system.html#resume"},{"doc":"gleam/otp/system","title":"suspend","content":"pub fn suspend(pid: Pid) -&gt; Nil\n Request an OTP compatible process to suspend, causing it to only handle\n system messages.\n\n For more information see the [Erlang documentation][1].\n\n [1]: https://erlang.org/doc/man/sys.html#suspend-1\n\nSynonyms:\ngleam/otp/system.suspend\ngleam/otp/system suspend","url":"gleam/otp/system.html#suspend"},{"doc":"gleam/otp/system","title":"gleam/otp/system","content":"","url":"gleam/otp/system.html"},{"doc":"gleam/otp/actor","title":"ErlangStartResult","content":"pub type ErlangStartResult =\n  Result(Pid, Dynamic)\n An Erlang supervisor compatible process start result.\n\n\nSynonyms:\ngleam/otp/actor.ErlangStartResult\ngleam/otp/actor ErlangStartResult","url":"gleam/otp/actor.html#ErlangStartResult"},{"doc":"gleam/otp/actor","title":"InitResult","content":"pub type InitResult(state, message) {\n  Ready(state: state, selector: Selector(message))\n  Failed(String)\n}\n The type used to indicate whether an actor has started successfully or not.\n\nReady(state: state, selector: Selector(message))\n The actor has successfully initialised. The actor can start handling\n messages and actor&#39;s channel sender can be returned to the parent\n process.\n\n\nFailed(String)\n The actor has failed to initialise. The actor shuts down and an error is\n returned to the parent process.\n\n\nSynonyms:\ngleam/otp/actor.InitResult\ngleam/otp/actor InitResult","url":"gleam/otp/actor.html#InitResult"},{"doc":"gleam/otp/actor","title":"Next","content":"pub type Next(message, state) {\n  Continue(state: state, selector: Option(Selector(message)))\n  Stop(ExitReason)\n}\n The type used to indicate what to do after handling a message.\n\nContinue(state: state, selector: Option(Selector(message)))\n Continue handling messages.\n\n An optional selector can be provided to changes the messages that the\n actor is handling. This replaces any selector that was previously given\n in the actor&#39;s `init` callback, or in any previous `Next` value.\n\n\nStop(ExitReason)\n Stop handling messages and shut down.\n\n\nSynonyms:\ngleam/otp/actor.Next\ngleam/otp/actor Next","url":"gleam/otp/actor.html#Next"},{"doc":"gleam/otp/actor","title":"Spec","content":"pub type Spec(state, msg) {\n  Spec(\n    init: fn() -&gt; InitResult(state, msg),\n    init_timeout: Int,\n    loop: fn(msg, state) -&gt; Next(msg, state),\n  )\n}\n This data structure holds all the values required by the `start_spec`\n function in order to create an actor.\n\n If you do not need to configure the initialisation behaviour of your actor\n consider using the `start` function.\n\nSpec(\n  init: fn() -&gt; InitResult(state, msg),\n  init_timeout: Int,\n  loop: fn(msg, state) -&gt; Next(msg, state),\n)\n\ninit\n&lt;p&gt;The initialisation functionality for the actor. This function is called\njust after the actor starts but before the channel sender is returned\nto the parent.&lt;/p&gt;\n&lt;p&gt;This function is used to ensure that any required data or state is\ncorrect. If this function returns an error it means that the actor has\nfailed to start and an error is returned to the parent.&lt;/p&gt;\n\ninit_timeout\n&lt;p&gt;How many milliseconds the &lt;code&gt;init&lt;/code&gt; function has to return before it is\nconsidered to have taken too long and failed.&lt;/p&gt;\n\nloop\n&lt;p&gt;This function is called to handle each message that the actor receives.&lt;/p&gt;\n\nSynonyms:\ngleam/otp/actor.Spec\ngleam/otp/actor Spec","url":"gleam/otp/actor.html#Spec"},{"doc":"gleam/otp/actor","title":"StartError","content":"pub type StartError {\n  InitTimeout\n  InitFailed(ExitReason)\n  InitCrashed(Dynamic)\n}\n\nInitTimeout\n\n\nInitFailed(ExitReason)\n\n\nInitCrashed(Dynamic)\n\n\nSynonyms:\ngleam/otp/actor.StartError\ngleam/otp/actor StartError","url":"gleam/otp/actor.html#StartError"},{"doc":"gleam/otp/actor","title":"StartResult","content":"pub type StartResult(msg) =\n  Result(Subject(msg), StartError)\n The result of starting a Gleam actor.\n\n This type is compatible with Gleam supervisors. If you wish to convert it\n to a type compatible with Erlang supervisors see the `ErlangStartResult`\n type and `erlang_start_result` function.\n\n\nSynonyms:\ngleam/otp/actor.StartResult\ngleam/otp/actor StartResult","url":"gleam/otp/actor.html#StartResult"},{"doc":"gleam/otp/actor","title":"call","content":"pub fn call(\n  subject: Subject(a),\n  make_message: fn(Subject(b)) -&gt; a,\n  timeout: Int,\n) -&gt; b\n Send a synchronous message and wait for a response from the receiving\n process.\n\n If a reply is not received within the given timeout then the sender process\n crashes. If you wish to receive a `Result` rather than crashing see the\n `process.try_call` function.\n\n This is a re-export of `process.call`, for the sake of convenience.\n\nSynonyms:\ngleam/otp/actor.call\ngleam/otp/actor call","url":"gleam/otp/actor.html#call"},{"doc":"gleam/otp/actor","title":"continue","content":"pub fn continue(state: a) -&gt; Next(b, a)\n\nSynonyms:\ngleam/otp/actor.continue\ngleam/otp/actor continue","url":"gleam/otp/actor.html#continue"},{"doc":"gleam/otp/actor","title":"send","content":"pub fn send(subject: Subject(a), msg: a) -&gt; Nil\n Send a message over a given channel.\n\n This is a re-export of `process.send`, for the sake of convenience.\n\nSynonyms:\ngleam/otp/actor.send\ngleam/otp/actor send","url":"gleam/otp/actor.html#send"},{"doc":"gleam/otp/actor","title":"start","content":"pub fn start(\n  state: a,\n  loop: fn(b, a) -&gt; Next(b, a),\n) -&gt; Result(Subject(b), StartError)\n Start an actor with a given initial state and message handling loop\n function.\n\n This function returns a `Result` but it will always be `Ok` so it is safe\n to use with `assert` if you are not starting this actor as part of a\n supervision tree.\n\n If you wish to configure the initialisation behaviour of a new actor see\n the `Spec` record and the `start_spec` function.\n\nSynonyms:\ngleam/otp/actor.start\ngleam/otp/actor start","url":"gleam/otp/actor.html#start"},{"doc":"gleam/otp/actor","title":"start_spec","content":"pub fn start_spec(\n  spec: Spec(a, b),\n) -&gt; Result(Subject(b), StartError)\n Start an actor from a given specification. If the actor&#39;s `init` function\n returns an error or does not return within `init_timeout` then an error is\n returned.\n\n If you do not need to specify the initialisation behaviour of your actor\n consider using the `start` function.\n\nSynonyms:\ngleam/otp/actor.start_spec\ngleam/otp/actor start_spec","url":"gleam/otp/actor.html#start_spec"},{"doc":"gleam/otp/actor","title":"to_erlang_start_result","content":"pub fn to_erlang_start_result(\n  res: Result(Subject(a), StartError),\n) -&gt; Result(Pid, Dynamic)\n Convert a Gleam actor start result into an Erlang supervisor-compatible\n process start result.\n\nSynonyms:\ngleam/otp/actor.to_erlang_start_result\ngleam/otp/actor to_erlang_start_result","url":"gleam/otp/actor.html#to_erlang_start_result"},{"doc":"gleam/otp/actor","title":"with_selector","content":"pub fn with_selector(\n  value: Next(a, b),\n  selector: Selector(a),\n) -&gt; Next(a, b)\n Provide a selector to change the messages that the actor is handling\n going forward. This replaces any selector that was previously given\n in the actor&#39;s `init` callback, or in any previous `Next` value.\n\nSynonyms:\ngleam/otp/actor.with_selector\ngleam/otp/actor with_selector","url":"gleam/otp/actor.html#with_selector"},{"doc":"gleam/otp/actor","title":"gleam/otp/actor","content":" This module provides the _Actor_ abstraction, one of the most common\n building blocks of Gleam OTP programs.\n \n An Actor is a process like any other BEAM process and can be be used to hold\n state, execute code, and communicate with other processes by sending and\n receiving messages. The advantage of using the actor abstraction over a bare\n process is that it provides a single interface for commonly needed\n functionality, including support for the [tracing and debugging\n features in OTP][erlang-sys].\n\n Gleam&#39;s Actor is similar to Erlang&#39;s `gen_server` and Elixir&#39;s `GenServer`\n but differs in that it offers a fully typed interface. This different API is\n why Gleam uses the name Actor rather than some variation of generic-server.\n\n [erlang-sys]: https://www.erlang.org/doc/man/sys.html\n\n ## Example\n\n An Actor can be used to create a client-server interaction between an Actor\n (the server) and other processes (the clients). In this example we have an\n Actor that works as a stack, allowing clients to push and pop elements.\n\n ```gleam\n pub fn main() {\n   // Start the actor with initial state of an empty list, and the\n   // `handle_message` callback function (defined below).\n   // We assert that it starts successfully.\n   // \n   // In real-world Gleam OTP programs we would likely write wrapper functions\n   // called `start`, `push` `pop`, `shutdown` to start and interact with the\n   // Actor. We are not doing that here for the sake of showing how the Actor \n   // API works.\n   let assert Ok(my_actor) = actor.start([], handle_message)\n \n   // We can send a message to the actor to push elements onto the stack.\n   process.send(my_actor, Push(&quot;Joe&quot;))\n   process.send(my_actor, Push(&quot;Mike&quot;))\n   process.send(my_actor, Push(&quot;Robert&quot;))\n \n   // The `Push` message expects no response, these messages are sent purely for\n   // the side effect of mutating the state held by the actor.\n   //\n   // We can also send the `Pop` message to take a value off of the actor&#39;s\n   // stack. This message expects a response, so we use `process.call` to send a\n   // message and wait until a reply is received.\n   //\n   // In this instance we are giving the actor 10 milliseconds to reply, if the\n   // `call` function doesn&#39;t get a reply within this time it will panic and\n   // crash the client process.\n   let assert Ok(&quot;Robert&quot;) = process.call(my_actor, Pop, 10)\n   let assert Ok(&quot;Mike&quot;) = process.call(my_actor, Pop, 10)\n   let assert Ok(&quot;Joe&quot;) = process.call(my_actor, Pop, 10)\n \n   // The stack is now empty, so if we pop again the actor replies with an error.\n   let assert Error(Nil) = process.call(my_actor, Pop, 10)\n \n   // Lastly, we can send a message to the actor asking it to shut down.\n   process.send(my_actor, Shutdown)\n }\n ```\n\n Here is the code that is used to implement this actor:\n\n ```gleam\n import gleam/erlang/process.{type Subject}\n import gleam/otp/actor\n\n // First step of implementing the stack Actor is to define the message type that\n // it can receive.\n //\n // The type of the elements in the stack is not fixed so a type parameter is used\n // for it instead of a concrete type such as `String` or `Int`.\n pub type Message(element) {\n   // The `Shutdown` message is used to tell the actor to stop.\n   // It is the simplest message type, it contains no data.\n   Shutdown\n \n   // The `Push` message is used to add a new element to the stack.\n   // It contains the item to add, the type of which is the `element`\n   // parameterised type.\n   Push(push: element)\n \n   // The `Pop` message is used to remove an element from the stack.\n   // It contains a `Subject`, which is used to send the response back to the\n   // message sender. In this case the reply is of type `Result(element, Nil)`.\n   Pop(reply_with: Subject(Result(element, Nil)))\n }\n \n // The last part is to implement the `handle_message` callback function.\n //\n // This function is called by the Actor for each message it receives.\n // Actor is single threaded only does one thing at a time, so it handles\n // messages sequentially and one at a time, in the order they are received.\n //\n // The function takes the message and the current state, and returns a data\n // structure that indicates what to do next, along with the new state.\n fn handle_message(\n  message: Message(e),\n  stack: List(e),\n ) -&gt; actor.Next(Message(e), List(e)) {\n   case message {\n     // For the `Shutdown` message we return the `actor.Stop` value, which causes\n     // the actor to discard any remaining messages and stop.\n     Shutdown -&gt; actor.Stop(process.Normal)\n \n     // For the `Push` message we add the new element to the stack and return\n     // `actor.continue` with this new stack, causing the actor to process any\n     // queued messages or wait for more.\n     Push(value) -&gt; {\n       let new_state = [value, ..stack]\n       actor.continue(new_state)\n     }\n \n     // For the `Pop` message we attempt to remove an element from the stack,\n     // sending it or an error back to the caller, before continuing.\n     Pop(client) -&gt;\n       case stack {\n         [] -&gt; {\n           // When the stack is empty we can&#39;t pop an element, so we send an\n           // error back.\n           process.send(client, Error(Nil))\n           actor.continue([])\n         }\n \n         [first, ..rest] -&gt; {\n           // Otherwise we send the first element back and use the remaining\n           // elements as the new state.\n           process.send(client, Ok(first))\n           actor.continue(rest)\n         }\n       }\n   }\n }\n ```","url":"gleam/otp/actor.html"},{"doc":"gleam/otp/intensity_tracker","title":"IntensityTracker","content":"pub opaque type IntensityTracker\n\n\nSynonyms:\ngleam/otp/intensity_tracker.IntensityTracker\ngleam/otp/intensity_tracker IntensityTracker","url":"gleam/otp/intensity_tracker.html#IntensityTracker"},{"doc":"gleam/otp/intensity_tracker","title":"TooIntense","content":"pub type TooIntense {\n  TooIntense\n}\n\nTooIntense\n\n\nSynonyms:\ngleam/otp/intensity_tracker.TooIntense\ngleam/otp/intensity_tracker TooIntense","url":"gleam/otp/intensity_tracker.html#TooIntense"},{"doc":"gleam/otp/intensity_tracker","title":"add_event","content":"pub fn add_event(\n  tracker: IntensityTracker,\n) -&gt; Result(IntensityTracker, TooIntense)\n\nSynonyms:\ngleam/otp/intensity_tracker.add_event\ngleam/otp/intensity_tracker add_event","url":"gleam/otp/intensity_tracker.html#add_event"},{"doc":"gleam/otp/intensity_tracker","title":"new","content":"pub fn new(\n  limit limit: Int,\n  period period: Int,\n) -&gt; IntensityTracker\n\nSynonyms:\ngleam/otp/intensity_tracker.new\ngleam/otp/intensity_tracker new","url":"gleam/otp/intensity_tracker.html#new"},{"doc":"gleam/otp/intensity_tracker","title":"trim_window","content":"pub fn trim_window(\n  events: List(Int),\n  now: Int,\n  period: Int,\n) -&gt; List(Int)\n\nSynonyms:\ngleam/otp/intensity_tracker.trim_window\ngleam/otp/intensity_tracker trim_window","url":"gleam/otp/intensity_tracker.html#trim_window"},{"doc":"gleam/otp/intensity_tracker","title":"gleam/otp/intensity_tracker","content":" The intensity tracker is used to monitor how frequently an event happens,\n erroring if it happens too many times within a period of time.","url":"gleam/otp/intensity_tracker.html"},{"doc":"gleam/otp/port","title":"Port","content":"pub type Port\n Ports are how code running on the Erlang virtual machine interacts with\n the outside world. Bytes of data can be sent to and read from ports,\n providing a form of message passing to an external program or resource.\n\n For more information on ports see the [Erlang ports documentation][1].\n\n [1]: https://erlang.org/doc/reference_manual/ports.html\n\n\nSynonyms:\ngleam/otp/port.Port\ngleam/otp/port Port","url":"gleam/otp/port.html#Port"},{"doc":"gleam/otp/port","title":"gleam/otp/port","content":"","url":"gleam/otp/port.html"},{"doc":"gleam/otp/static_supervisor","title":"AutoShutdown","content":"pub type AutoShutdown {\n  Never\n  AnySignificant\n  AllSignificant\n}\n A supervisor can be configured to automatically shut itself down with exit\n reason shutdown when significant children terminate with the auto_shutdown\n key in the above map.\nNever\n Automic shutdown is disabled. This is the default setting.\n\n With auto_shutdown set to never, child specs with the significant flag\n set to true are considered invalid and will be rejected.\n\nAnySignificant\n The supervisor will shut itself down when any significant child\n terminates, that is, when a transient significant child terminates\n normally or when a temporary significant child terminates normally or\n abnormally.\n\nAllSignificant\n The supervisor will shut itself down when all significant children have\n terminated, that is, when the last active significant child terminates.\n The same rules as for any_significant apply.\n\nSynonyms:\ngleam/otp/static_supervisor.AutoShutdown\ngleam/otp/static_supervisor AutoShutdown","url":"gleam/otp/static_supervisor.html#AutoShutdown"},{"doc":"gleam/otp/static_supervisor","title":"Builder","content":"pub opaque type Builder\n\n\nSynonyms:\ngleam/otp/static_supervisor.Builder\ngleam/otp/static_supervisor Builder","url":"gleam/otp/static_supervisor.html#Builder"},{"doc":"gleam/otp/static_supervisor","title":"ChildBuilder","content":"pub opaque type ChildBuilder\n\n\nSynonyms:\ngleam/otp/static_supervisor.ChildBuilder\ngleam/otp/static_supervisor ChildBuilder","url":"gleam/otp/static_supervisor.html#ChildBuilder"},{"doc":"gleam/otp/static_supervisor","title":"ChildType","content":"pub type ChildType {\n  Worker(shutdown_ms: Int)\n  Supervisor\n}\n\nWorker(shutdown_ms: Int)\n\nshutdown_ms\n&lt;p&gt;The number of milliseconds the child is given to shut down. The\nsupervisor tells the child process to terminate by calling\n&lt;code&gt;exit(Child,shutdown)&lt;/code&gt; and then wait for an exit signal with reason\nshutdown back from the child process. If no exit signal is received\nwithin the specified number of milliseconds, the child process is\nunconditionally terminated using &lt;code&gt;exit(Child,kill)&lt;/code&gt;.&lt;/p&gt;\n\nSupervisor\n\n\nSynonyms:\ngleam/otp/static_supervisor.ChildType\ngleam/otp/static_supervisor ChildType","url":"gleam/otp/static_supervisor.html#ChildType"},{"doc":"gleam/otp/static_supervisor","title":"Restart","content":"pub type Restart {\n  Permanent\n  Transient\n  Temporary\n}\n Restart defines when a terminated child process must be restarted. \nPermanent\n A permanent child process is always restarted.\n\nTransient\n A transient child process is restarted only if it terminates abnormally,\n that is, with another exit reason than `normal`, `shutdown`, or\n `{shutdown,Term}`.\n\nTemporary\n A temporary child process is never restarted (even when the supervisor&#39;s\n restart strategy is `RestForOne` or `OneForAll` and a sibling&#39;s death\n causes the temporary process to be terminated).\n\nSynonyms:\ngleam/otp/static_supervisor.Restart\ngleam/otp/static_supervisor Restart","url":"gleam/otp/static_supervisor.html#Restart"},{"doc":"gleam/otp/static_supervisor","title":"Strategy","content":"pub type Strategy {\n  OneForOne\n  OneForAll\n  RestForOne\n}\n\nOneForOne\n If one child process terminates and is to be restarted, only that child\n process is affected. This is the default restart strategy.\n\nOneForAll\n If one child process terminates and is to be restarted, all other child\n processes are terminated and then all child processes are restarted.\n\nRestForOne\n If one child process terminates and is to be restarted, the &#39;rest&#39; of the\n child processes (that is, the child processes after the terminated child\n process in the start order) are terminated. Then the terminated child\n process and all child processes after it are restarted.\n\nSynonyms:\ngleam/otp/static_supervisor.Strategy\ngleam/otp/static_supervisor Strategy","url":"gleam/otp/static_supervisor.html#Strategy"},{"doc":"gleam/otp/static_supervisor","title":"add","content":"pub fn add(builder: Builder, child: ChildBuilder) -&gt; Builder\n Add a child to the supervisor.\nSynonyms:\ngleam/otp/static_supervisor.add\ngleam/otp/static_supervisor add","url":"gleam/otp/static_supervisor.html#add"},{"doc":"gleam/otp/static_supervisor","title":"auto_shutdown","content":"pub fn auto_shutdown(\n  builder: Builder,\n  value: AutoShutdown,\n) -&gt; Builder\n A supervisor can be configured to automatically shut itself down with\n exit reason shutdown when significant children terminate.\nSynonyms:\ngleam/otp/static_supervisor.auto_shutdown\ngleam/otp/static_supervisor auto_shutdown","url":"gleam/otp/static_supervisor.html#auto_shutdown"},{"doc":"gleam/otp/static_supervisor","title":"new","content":"pub fn new(strategy strategy: Strategy) -&gt; Builder\n\nSynonyms:\ngleam/otp/static_supervisor.new\ngleam/otp/static_supervisor new","url":"gleam/otp/static_supervisor.html#new"},{"doc":"gleam/otp/static_supervisor","title":"restart","content":"pub fn restart(\n  child: ChildBuilder,\n  restart: Restart,\n) -&gt; ChildBuilder\n When the child is to be restarted. See the `Restart` documentation for\n more.\n\n The default value for significance is `Permenent`.\nSynonyms:\ngleam/otp/static_supervisor.restart\ngleam/otp/static_supervisor restart","url":"gleam/otp/static_supervisor.html#restart"},{"doc":"gleam/otp/static_supervisor","title":"restart_tolerance","content":"pub fn restart_tolerance(\n  builder: Builder,\n  intensity intensity: Int,\n  period period: Int,\n) -&gt; Builder\n To prevent a supervisor from getting into an infinite loop of child\n process terminations and restarts, a maximum restart intensity is\n defined using two integer values specified with keys intensity and\n period in the above map. Assuming the values MaxR for intensity and MaxT\n for period, then, if more than MaxR restarts occur within MaxT seconds,\n the supervisor terminates all child processes and then itself. The\n termination reason for the supervisor itself in that case will be\n shutdown. \n\n Intensity defaults to 1 and period defaults to 5.\nSynonyms:\ngleam/otp/static_supervisor.restart_tolerance\ngleam/otp/static_supervisor restart_tolerance","url":"gleam/otp/static_supervisor.html#restart_tolerance"},{"doc":"gleam/otp/static_supervisor","title":"significant","content":"pub fn significant(\n  child: ChildBuilder,\n  significant: Bool,\n) -&gt; ChildBuilder\n This defines if a child is considered significant for automatic\n self-shutdown of the supervisor.\n\n You most likely do not want to consider any children significant.\n\n This will be ignored if the supervisor auto shutdown is set to `Never`,\n which is the default.\n\n The default value for significance is `False`.\nSynonyms:\ngleam/otp/static_supervisor.significant\ngleam/otp/static_supervisor significant","url":"gleam/otp/static_supervisor.html#significant"},{"doc":"gleam/otp/static_supervisor","title":"start_link","content":"pub fn start_link(builder: Builder) -&gt; Result(Pid, Dynamic)\n\nSynonyms:\ngleam/otp/static_supervisor.start_link\ngleam/otp/static_supervisor start_link","url":"gleam/otp/static_supervisor.html#start_link"},{"doc":"gleam/otp/static_supervisor","title":"supervisor_child","content":"pub fn supervisor_child(\n  id id: String,\n  run starter: fn() -&gt; Result(Pid, a),\n) -&gt; ChildBuilder\n A special child that is a supervisor itself.\n\n id is used to identify the child specification internally by the\n supervisor.\n Notice that this identifier on occations has been called &quot;name&quot;. As far\n as possible, the terms &quot;identifier&quot; or &quot;id&quot; are now used but to keep\n backward compatibility, some occurences of &quot;name&quot; can still be found, for\n example in error messages.\n\nSynonyms:\ngleam/otp/static_supervisor.supervisor_child\ngleam/otp/static_supervisor supervisor_child","url":"gleam/otp/static_supervisor.html#supervisor_child"},{"doc":"gleam/otp/static_supervisor","title":"timeout","content":"pub fn timeout(child: ChildBuilder, ms ms: Int) -&gt; ChildBuilder\n This defines the amount of milliseconds a child has to shut down before\n being brutal killed by the supervisor.\n\n If not set the default for a child is 5000ms.\n\n This will be ignored if the child is a supervisor itself.\n\nSynonyms:\ngleam/otp/static_supervisor.timeout\ngleam/otp/static_supervisor timeout","url":"gleam/otp/static_supervisor.html#timeout"},{"doc":"gleam/otp/static_supervisor","title":"worker_child","content":"pub fn worker_child(\n  id id: String,\n  run starter: fn() -&gt; Result(Pid, a),\n) -&gt; ChildBuilder\n A regular child that is not also a supervisor.\n\n id is used to identify the child specification internally by the\n supervisor.\n Notice that this identifier on occations has been called &quot;name&quot;. As far\n as possible, the terms &quot;identifier&quot; or &quot;id&quot; are now used but to keep\n backward compatibility, some occurences of &quot;name&quot; can still be found, for\n example in error messages.\n\nSynonyms:\ngleam/otp/static_supervisor.worker_child\ngleam/otp/static_supervisor worker_child","url":"gleam/otp/static_supervisor.html#worker_child"},{"doc":"gleam/otp/static_supervisor","title":"gleam/otp/static_supervisor","content":" Bindings to Erlang/OTP&#39;s `supervisor` module.\n\n For further detail see the Erlang documentation:\n &lt;https://www.erlang.org/doc/apps/stdlib/supervisor.html&gt;.\n\n # Example\n\n ```gleam\n import gleam/erlang/process.{type Pid}\n import gleam/otp/static_supervisor as sup\n \n pub fn start_supervisor() {\n   sup.new(sup.OneForOne)\n   |&gt; sup.add(sup.worker_child(&quot;db&quot;, start_database_connection))\n   |&gt; sup.add(sup.worker_child(&quot;workers&quot;, start_workers))\n   |&gt; sup.add(sup.worker_child(&quot;web&quot;, start_http_server))\n   |&gt; sup.start_link\n }\n ```","url":"gleam/otp/static_supervisor.html"},{"doc":"gleam/otp/supervisor","title":"ApplicationStartMode","content":"pub type ApplicationStartMode {\n  Normal\n  Takeover(Node)\n  Failover(Node)\n}\n A type used to describe the situation in which an Erlang based application\n is starting.\n\n For more information see the [Erlang distributed application\n documentation][1] and the Learn You Some Erlang chapter on [distributed\n applications][2].\n\n [1]: https://erlang.org/doc/design_principles/distributed_applications.html\n [2]: https://learnyousomeerlang.com/distributed-otp-applications\n\nNormal\n\n\nTakeover(Node)\n\n\nFailover(Node)\n\n\nSynonyms:\ngleam/otp/supervisor.ApplicationStartMode\ngleam/otp/supervisor ApplicationStartMode","url":"gleam/otp/supervisor.html#ApplicationStartMode"},{"doc":"gleam/otp/supervisor","title":"ApplicationStop","content":"pub type ApplicationStop\n\n\nSynonyms:\ngleam/otp/supervisor.ApplicationStop\ngleam/otp/supervisor ApplicationStop","url":"gleam/otp/supervisor.html#ApplicationStop"},{"doc":"gleam/otp/supervisor","title":"ChildSpec","content":"pub opaque type ChildSpec(msg, argument, returning)\n This type contains all the information required to start a new child and\n add it to the `Children`.\n\n This is typically created with the `worker` function.\n\n\nSynonyms:\ngleam/otp/supervisor.ChildSpec\ngleam/otp/supervisor ChildSpec","url":"gleam/otp/supervisor.html#ChildSpec"},{"doc":"gleam/otp/supervisor","title":"Children","content":"pub opaque type Children(argument)\n This type represents the starting children of a supervisor within the\n `init` function.\n\n\nSynonyms:\ngleam/otp/supervisor.Children\ngleam/otp/supervisor Children","url":"gleam/otp/supervisor.html#Children"},{"doc":"gleam/otp/supervisor","title":"ErlangStartResult","content":"pub type ErlangStartResult =\n  actor.ErlangStartResult\n An Erlang supervisor compatible process start result.\n\n\nSynonyms:\ngleam/otp/supervisor.ErlangStartResult\ngleam/otp/supervisor ErlangStartResult","url":"gleam/otp/supervisor.html#ErlangStartResult"},{"doc":"gleam/otp/supervisor","title":"Message","content":"pub opaque type Message\n\n\nSynonyms:\ngleam/otp/supervisor.Message\ngleam/otp/supervisor Message","url":"gleam/otp/supervisor.html#Message"},{"doc":"gleam/otp/supervisor","title":"Spec","content":"pub type Spec(argument, return) {\n  Spec(\n    argument: argument,\n    max_frequency: Int,\n    frequency_period: Int,\n    init: fn(Children(argument)) -&gt; Children(return),\n  )\n}\n This data structure holds all the values required by the `start_spec`\n function in order to create an supervisor.\n\n If you do not need to configure the behaviour of your supervisor consider\n using the `start` function.\n\nSpec(\n  argument: argument,\n  max_frequency: Int,\n  frequency_period: Int,\n  init: fn(Children(argument)) -&gt; Children(return),\n)\n\n\nSynonyms:\ngleam/otp/supervisor.Spec\ngleam/otp/supervisor Spec","url":"gleam/otp/supervisor.html#Spec"},{"doc":"gleam/otp/supervisor","title":"StartResult","content":"pub type StartResult(msg) =\n  actor.StartResult(msg)\n The result of starting a Gleam actor.\n\n This type is compatible with Gleam supervisors. If you wish to convert it\n to a type compatible with Erlang supervisors see the `ErlangStartResult`\n type and `erlang_start_result` function.\n\n\nSynonyms:\ngleam/otp/supervisor.StartResult\ngleam/otp/supervisor StartResult","url":"gleam/otp/supervisor.html#StartResult"},{"doc":"gleam/otp/supervisor","title":"add","content":"pub fn add(\n  children: Children(a),\n  child_spec: ChildSpec(b, a, c),\n) -&gt; Children(c)\n Add a child to the collection of children of the supervisor\n\n This function starts the child from the child spec.\n\nSynonyms:\ngleam/otp/supervisor.add\ngleam/otp/supervisor add","url":"gleam/otp/supervisor.html#add"},{"doc":"gleam/otp/supervisor","title":"application_stopped","content":"pub fn application_stopped() -&gt; ApplicationStop\n\nSynonyms:\ngleam/otp/supervisor.application_stopped\ngleam/otp/supervisor application_stopped","url":"gleam/otp/supervisor.html#application_stopped"},{"doc":"gleam/otp/supervisor","title":"returning","content":"pub fn returning(\n  child: ChildSpec(a, b, c),\n  updater: fn(b, Subject(a)) -&gt; d,\n) -&gt; ChildSpec(a, b, d)\n As each child is added to a supervisors children a new argument is prepared\n with which to start the next child. By default argument is the same as the\n previous argument, but this function can be used to change it to something\n else by passing a function that takes the previous argument and the sender\n of the previous child.\n\nSynonyms:\ngleam/otp/supervisor.returning\ngleam/otp/supervisor returning","url":"gleam/otp/supervisor.html#returning"},{"doc":"gleam/otp/supervisor","title":"start","content":"pub fn start(\n  init: fn(Children(Nil)) -&gt; Children(a),\n) -&gt; Result(Subject(Message), StartError)\n Start a supervisor from a given `init` function.\n\n The init argument passed to children will be `Nil` and the maximum restart\n intensity will be 1 restart per 5 seconds (the same as the default for\n [Erlang supervisors][erl-sup]). If you wish to specify these values, see\n the `start_spec` function and the `Spec` type.\n\n [erl-sup]: https://www.erlang.org/doc/design_principles/sup_princ.html#maximum-restart-intensity\n\n ## Examples\n \n \n let worker = worker(my_actor.start)\n\n let children = fn(children) {\n   children\n   |&gt; add(worker)\n   |&gt; add(worker)\n }\n\n start(children)\n \n\nSynonyms:\ngleam/otp/supervisor.start\ngleam/otp/supervisor start","url":"gleam/otp/supervisor.html#start"},{"doc":"gleam/otp/supervisor","title":"start_spec","content":"pub fn start_spec(\n  spec: Spec(a, b),\n) -&gt; Result(Subject(Message), StartError)\n Start a supervisor from a given specification.\n\n\n ## Examples\n \n \n let worker = worker(my_actor.start)\n\n let children = fn(children) {\n   children\n   |&gt; add(worker)\n   |&gt; add(worker)\n }\n\n start_spec(Spec(\n   argument: initial_state,\n   frequency_period: 1,\n   max_frequency: 5,\n   init: children,\n ))\n \n\nSynonyms:\ngleam/otp/supervisor.start_spec\ngleam/otp/supervisor start_spec","url":"gleam/otp/supervisor.html#start_spec"},{"doc":"gleam/otp/supervisor","title":"supervisor","content":"pub fn supervisor(\n  start: fn(a) -&gt; Result(Subject(b), StartError),\n) -&gt; ChildSpec(b, a, a)\n Prepare a new supervisor type child.\n\n If you wish to prepare a new non-supervisor type child see the `worker`\n function.\n\n If you wish to change the type of the argument for later children see the\n `returning` function.\n\n Note: Gleam supervisors do not yet support different shutdown periods per\n child so this function is currently identical in behaviour to `worker`. It is\n recommended to use this function for supervisor children nevertheless so the\n correct shut down behaviour is used in later releases of this library.\n\nSynonyms:\ngleam/otp/supervisor.supervisor\ngleam/otp/supervisor supervisor","url":"gleam/otp/supervisor.html#supervisor"},{"doc":"gleam/otp/supervisor","title":"to_erlang_start_result","content":"pub fn to_erlang_start_result(\n  res: Result(Subject(a), StartError),\n) -&gt; Result(Pid, Dynamic)\n Convert a Gleam actor start result into an Erlang supervisor compatible\n process start result.\n\nSynonyms:\ngleam/otp/supervisor.to_erlang_start_result\ngleam/otp/supervisor to_erlang_start_result","url":"gleam/otp/supervisor.html#to_erlang_start_result"},{"doc":"gleam/otp/supervisor","title":"worker","content":"pub fn worker(\n  start: fn(a) -&gt; Result(Subject(b), StartError),\n) -&gt; ChildSpec(b, a, a)\n Prepare a new worker type child.\n\n If you wish to prepare a new supervisor type child see the `supervisor`\n function.\n\n If you wish to change the type of the argument for later children see the\n `returning` function.\n\n ## Examples\n\n \n worker(fn(argument) {\n  my_actor.start(argument)\n })\n  \n\nSynonyms:\ngleam/otp/supervisor.worker\ngleam/otp/supervisor worker","url":"gleam/otp/supervisor.html#worker"},{"doc":"gleam/otp/supervisor","title":"gleam/otp/supervisor","content":" A supervisor that can pass state from older children to younger ones.\n\n If you don&#39;t need this consider using the `gleam/otp/static_supervisor`\n module instead.","url":"gleam/otp/supervisor.html"},{"doc":"gleam/otp/task","title":"AwaitError","content":"pub type AwaitError {\n  Timeout\n  Exit(reason: Dynamic)\n}\n\nTimeout\n\n\nExit(reason: Dynamic)\n\n\nSynonyms:\ngleam/otp/task.AwaitError\ngleam/otp/task AwaitError","url":"gleam/otp/task.html#AwaitError"},{"doc":"gleam/otp/task","title":"Task","content":"pub opaque type Task(value)\n\n\nSynonyms:\ngleam/otp/task.Task\ngleam/otp/task Task","url":"gleam/otp/task.html#Task"},{"doc":"gleam/otp/task","title":"async","content":"pub fn async(work: fn() -&gt; a) -&gt; Task(a)\n Spawn a task process that calls a given function in order to perform some\n work. The result of this function is send back to the parent and can be\n received using the `await` function.\n\n See the top level module documentation for more information on async/await.\n\nSynonyms:\ngleam/otp/task.async\ngleam/otp/task async","url":"gleam/otp/task.html#async"},{"doc":"gleam/otp/task","title":"await","content":"pub fn await(task: Task(a), timeout: Int) -&gt; a\n Wait for the value computed by a task.\n\n If the a value is not received before the timeout has elapsed or if the\n task process crashes then this function crashes.\n\nSynonyms:\ngleam/otp/task.await\ngleam/otp/task await","url":"gleam/otp/task.html#await"},{"doc":"gleam/otp/task","title":"await_forever","content":"pub fn await_forever(task: Task(a)) -&gt; a\n Wait endlessly for the value computed by a task.\n\n Be Careful! Like `try_await_forever`, this function does not return until there is a value to\n receive.\n\n If the task process crashes then this function crashes.\n\nSynonyms:\ngleam/otp/task.await_forever\ngleam/otp/task await_forever","url":"gleam/otp/task.html#await_forever"},{"doc":"gleam/otp/task","title":"try_await","content":"pub fn try_await(\n  task: Task(a),\n  timeout: Int,\n) -&gt; Result(a, AwaitError)\n Wait for the value computed by a task.\n\n If the a value is not received before the timeout has elapsed or if the\n task process crashes then an error is returned.\n\nSynonyms:\ngleam/otp/task.try_await\ngleam/otp/task try_await","url":"gleam/otp/task.html#try_await"},{"doc":"gleam/otp/task","title":"try_await_forever","content":"pub fn try_await_forever(task: Task(a)) -&gt; Result(a, AwaitError)\n Wait endlessly for the value computed by a task.\n\n Be Careful! This function does not return until there is a value to\n receive. If a value is not received then the process will be stuck waiting\n forever.\n\nSynonyms:\ngleam/otp/task.try_await_forever\ngleam/otp/task try_await_forever","url":"gleam/otp/task.html#try_await_forever"},{"doc":"gleam/otp/task","title":"gleam/otp/task","content":" A task is a kind of process that performs a single task and then shuts\n down. Commonly tasks are used to convert sequential code into concurrent\n code by performing computation in another process.\n\n ```gleam\n let task = task.async(fn() { do_some_work() })\n let value = do_some_other_work()\n value + task.await(task, 100)\n ```\n\n Tasks spawned with async can be awaited on by their caller process (and\n only their caller) as shown in the example above. They are implemented by\n spawning a process that sends a message to the caller once the given\n computation is performed.\n\n There are two important things to consider when using `async`:\n\n 1. If you are using async tasks, you must await a reply as they are always\n    sent.\n\n 2. async tasks link the caller and the spawned process. This means that,\n    if the caller crashes, the task will crash too and vice-versa. This is\n    on purpose: if the process meant to receive the result no longer\n    exists, there is no purpose in completing the computation.\n\n This module is inspired by Elixir&#39;s [Task module][1].\n\n [1]: https://hexdocs.pm/elixir/master/Task.html\n","url":"gleam/otp/task.html"}]);